/*
 * CrudContainer
 * Copyright (C) 2011 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.filter.lucene;

import dk.apaq.filter.Filter;
import dk.apaq.filter.core.AndFilter;
import dk.apaq.filter.core.CompareFilter;
import dk.apaq.filter.core.LikeFilter;
import dk.apaq.filter.core.NotFilter;
import dk.apaq.filter.core.OrFilter;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.BooleanClause.Occur;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.FuzzyQuery;
import org.apache.lucene.search.PrefixQuery;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.TermQuery;

/**
 * A Filter Translator for Lucene. The translator converts the Filter,
 * Sorter and Limit objects to matching object.
 */
public class FilterTranslatorForLucene {

    private static final FilterTranslatorForLucene SINGLETON = new FilterTranslatorForLucene();

    public static FilterTranslatorForLucene getTranslator() {
        return SINGLETON;
    }

    private FilterTranslatorForLucene() { /* EMPTY */ }

    public Query translate(Filter filter) {
        if (filter instanceof LikeFilter) {
            LikeFilter cf = (LikeFilter) filter;
            if (cf.getValue() == null) {
                return null;
            }

            BooleanQuery query = new BooleanQuery();
            String searchString = cf.getValue().trim();
            searchString = searchString.replace(" *", "");

            String[] valueArray = searchString.split(" ");

            for (String value : valueArray) {
                boolean usePrefixTerm = false;
                value = value.trim();
                if (value.startsWith("-") || value.startsWith("+")) {
                    continue;
                }

                if (value.endsWith("*")) {
                    usePrefixTerm = true;
                    value = value.substring(0, value.length() - 1);
                }

                if ("".equals(value) || "&".equals(value) || "*".equals(value)) {
                    continue;
                }

                //Lucene by default indexes as lowercase.
                value = value.toLowerCase();

                BooleanQuery currentQuery = new BooleanQuery();
                Term term = new Term(cf.getPropertyId(), value);

                if (!cf.isPrecise()) {
                    currentQuery.add(new FuzzyQuery(term, cf.getPrecision()), Occur.SHOULD);
                } else {
                    currentQuery.add(new TermQuery(term), Occur.SHOULD);
                }

                if (usePrefixTerm) {
                    currentQuery.add(new PrefixQuery(term), Occur.SHOULD);
                }

                query.add(currentQuery, Occur.MUST);
            }

            return query;
        }

        if (filter instanceof CompareFilter) {
            CompareFilter cf = (CompareFilter) filter;
            switch (cf.getCompareType()) {
                case Equals:
                    return new TermQuery(new Term(cf.getPropertyId(), cf.getValue().toString()));

                //TODO
                default:
                    throw new UnsupportedOperationException("Not supported by Translator");
            }
        }

        if (filter instanceof AndFilter) {
            AndFilter af = (AndFilter) filter;

            BooleanQuery query = new BooleanQuery();
            for (Filter current : af.getFilters()) {
                Query subQuery = translate(current);
                if (subQuery != null) {
                    query.add(subQuery, Occur.MUST);
                }
            }
            return query;
        }

        if (filter instanceof OrFilter) {
            OrFilter af = (OrFilter) filter;
            StringBuilder sb = new StringBuilder();

            BooleanQuery query = new BooleanQuery();
            for (Filter current : af.getFilters()) {
                Query subQuery = translate(current);
                if (subQuery != null) {
                    query.add(subQuery, Occur.SHOULD);
                }
            }
            return query;
        }

        if (filter instanceof NotFilter) {
            NotFilter af = (NotFilter) filter;
            BooleanQuery query = new BooleanQuery();
            Query subQuery = translate(af.getFilterElement());
            if (subQuery != null) {
                query.add(subQuery, Occur.MUST_NOT);
            }
            return query;
        }

        return null;
    }
}
